@@ -223,7 +223,7 @@ gst_marble_class_init (GstMarbleClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gst_element_class_set_metadata (gstelement_class,
+  gst_element_class_set_static_metadata (gstelement_class,
       "marble",
       "Transform/Effect/Video",
       "Applies a marbling effect to the image",
